// import QtQuick 2.0

import MuseScore 3.0

//--------------------------------------------------------------------------

/* Ce plugin ajoute le numéro des touches pour accordéon diatonique 

    afin de créer une forme très simplifiée de tablature

  

  Auteur : Jean-Michel Bencetti

  Version courrante : 1.03

  Date : v1.00 : 2019-06-13 : développement initial

            v1.02 : 2019-09-02 : tient compte des accords main gauche pour proposer les notes en tiré ou en poussé

            v1.03 : 2019-10-11 : ajoute la possibilité de ne traiter que des mesures sélectionnées

  

  Description version v1.02 :

    Pour les accords main gauche A, Am, D, Dm, seules les touches en tirées sont proposées

    Pour les accords main gauche E, Em, E7, C, seules les touches en poussé sont proposées

    Pour les accords main gauche G et F, les deux numéros de touches sont proposées lorsqu'elles existes

    Les notes altérées (sauf F#) ne sont pas proposées car trop de plan de claviers différents existent

    Pour la note G, les deux propositions sont faites sur le premier et deuxième rang

  

  Après le passage du plugin, il reste donc à faire le ménage pour supprimer les numéros de touches en trop

  pour les accords F et G et sur les notes G main droite

  

  Description version v1.03 : 

  - pour limiter le travail du plugin, il est possible de sélectionner les mesures à traiter.

  - sans sélection, le plugin travaille sur touite la partition sanf la dernière portée.

  - la dernière portée n'est pas traitée car elle est sencée être en clé de Fa avec des Basses et des Accords.

  - pour traiter quand même la dernière portée, il suffit de la sélectionner.

  

  

  ----------------------------------------------------------------------------*/


MuseScore {

      menuPath: "Plugins.Mini Diatonic"

      description: qsTr("Tablatures simplifiées pour accordéons diatoniques")

      requiresScore: true

      version: "1.03"

      

    property var noteName: [

      "C","C#","D","D#","E","F","F#","G","G#","A","A#","B",

      "c","c#","d","d#","e","f","f#","g","g#","a","a#","b",

      "c'" , "c'#" , "d'" , "d'#" , "e'" , "f'" ,

       "f'#" , "g'" , "g'#" , "a'" , "a'#" , "b'" , "c''"

    ]

   // Numéro des boutons à appuyer. Si plusieurs boutons possible, il y a un  / entre les numéros

/*   property var boutonPousse3rng :  [

      " ", //C

      " ", //C# 

      "2",//D 

      " ", //D#

      "1'",//E

      "X", //F

      "X", //F# 

      "3/2'", //G

       "1''", //G#

       "2''", //A

       " ", //A#

       "4", //B

       "3'",  //C

       " ", //C#

       "5", //D

       "3''", //D#  / Eb

       "4'", //E

       "X", //F

       "X", //F#

       "6/5'", //G

       "4''",  //G#

       "5''", //A

       " ",  // A# / Bb

       "7", //B

       "6'", //C

       " ",  //C# 

       "8", //D

       "6''", //D# - Eb

       "7'",  //E

       "X", //F

       "X", //F#

       "9/8'", //G

       "7''", //G#

        "8''", //A 

        " ", //A# - Bb

        "10", //B

        "9'", //C

        " ",  //C#

        "11", //D

        "9''",  //D# - Eb

        "10'", //E

        "X", //F

        "X", //F#

         "11", //G

         "", //G#

          "10''", //A

          " ", //A# - Bb

          " " //B

    ] */

       property var boutonPousse :  [

      " ", //C

      " ", //C# 

      "2",//D 

      " ", //D#

      "1'",//E

      " ", //F

      " ", //F# 

      "3", //G

       " ", //G#

       " ", //A

       " ", //A#

       "4", //B

       "3'",  //C

       " ", //C#

       "5", //D

       " ", //D#  / Eb

       "4'", //E

       " ", //F

       " ", //F#

       "6/5'", //G

       " ",  //G#

       " ", //A

       " ",  // A# / Bb

       "7", //B

       "6'", //C

       " ",  //C# 

       "8", //D

       " ", //D# - Eb

       "7'",  //E

       " ", //F

       " ", //F#

       "9/8'", //G

       " ", //G#

        " ", //A 

        " ", //A# - Bb

        "10", //B

        "9'", //C

        " ",  //C#

        "11", //D

        " ",  //D# - Eb

        "10'", //E

        " ", //F

        " ", //F#

         " ", //G

         " ", //G#

          " ", //A

          " ", //A# - Bb

          " " //B

    ]

 property var boutonTire :  [

    " ", //C

    " ", //C#

    "1", //D

    " ",  //D#

    " ", //E

    "1'",  //F

    "2", //F#

    "1''", //G 

    " ", // G#

    "3", //A

    " ",  //A# - Bb

    "2'", //B

    "4 ", //C

    "3'", //C#

    "3'", //D

    " ", //D#

    "5 ",  //E

    "4'", //F

    "6", //F# 

    " ",//G

    " ", //G#

    "5'/7",//A

    " ", //A# - Bb

    "6'", //B

    "8", //C

    " ", //C#

    "7'", //D

    " ", //D# - Eb

    "9", //E 

    "8'", //F

    "10", //F#

    " ", //G

    " ",  //G#

    "9'/11",  //A

    " ", //A# - Bb

    "10'", // B

    " ", //C

    " ", //C#

    " ", //D

    "11'", //D# - Eb

    "10'", //E

    " ", //F

    " ", //F#

    " ", //G

    " ", //G#

    "11",//A

    " ", //A# - Bb

     " " //C

    ]

 /*   property var boutonTire3rng :  [

    " ", //C

    " ", //C#

    "1", //D

    " ",  //D#

    " ", //E

    "1'",  //F

    "2", //F#

    "1''", //G 

    " ", // G#

    "3", //A

    "2''",  //A# - Bb

    "2'", //B

    "4 ", //C

    "3'", //C#

    "3'", //D

    " ", //D#

    "5 ",  //E

    "4'", //F

    "6", //F# 

    "4''",//G

    " ", //G#

    "7",//A

    "5''", //A# - Bb

    "6'", //B

    "8", //C

    "6''", //C#

    "7'", //D

    " ", //D# - Eb

    "9", //E 

    "8'", //F

    "10", //F#

    "7''", //G

    " ",  //G#

    "9'",  //A

    "8''", //A# - Bb

    "10'", // B

    "9''", //C

    " ", //C#

    "12", //D

    "11'", //D# - Eb

    "10'", //E

    " ", //F

    " ", //F#

    " ", //G

    " ", //G#

    "11",//A

    " ", //A# - Bb

     " " //C

    ] */

 // ------------------------------------------------------------------------------

 // Cette fonction ajoute le numéro de la touche à actionner en fonction de l'accord main gauche

 // Entrée : curseur positionné à l'endroit où il faut insérer le numéro de la touche

 //              note à traiter, cette fonction ne traite qu'une seile note à la fois

 //              le dernier accord main gauche rencontré pour choisir entre tiré et poussé lorsque c'est possible

 // Si la note n'existe pas en poussé mais qu'elle existe en tiré, celle-ci est proposée quelque soit l'accord (A, F, F#))

 // et réciproquement

 //------------------------------------------------------------------------------

 function addTouche(cursor, note, accord) {

        var textPousse = newElement(Element.STAFF_TEXT)

        var textTire      = newElement(Element.STAFF_TEXT)

        var pitch = note.pitch

        var index = pitch - 48

        if (index >= 0 && index < boutonTire.length) {

           textPousse.text = boutonPousse[index]

           textTire.text      = "<u>" + boutonTire[index] + "</u>"  // Les numéros de touche en tiré sont soulignés

           

           if ((accord[0] == "A") || (accord[0] == "D")) if (boutonTire[index] != " ") textPousse.text      = "";

           if ((accord[0] == "E") || (accord[0] == "C"))  if (boutonPousse[index] != " ") textTire.text      = "";

                      

           //console.log("Note = " + noteName[index] + " Pousse " + textPousse.text + " - Tire " + textTire.text)

           textPousse.offsetY = 10

           textTire.offsetY = 10

           if (!(textPousse === "")) cursor.add(textPousse)

           if (!(textTire === ""))      cursor.add(textTire)

        

        }

    }

 

      

 onRun: {

       if (!curScore) Qt.quit();   // Si pas de partition courrante, sortie du plugin

       if (typeof curScore === 'undefined')  Qt.quit();

           

      // stave = portée musicale

      var myScore = curScore,                        // Partition en cours

           cursor = myScore.newCursor(),        // Fabrique un curseur pour se déplacer dans les mesures

           startStaff,                                         // Début de partition ou début de sélection

           endStaff,                                          // Fin de partition ou fin de sélection

           endTick,                                          // Numéro du dernier élément de la partition ou de la sélection

           staff = 0,                                         // Compteur sur le nombre de portée de la partition

           accordMg,                                       // Détermine si on est en Poussé ou en Tiré lorsque c'est possible

           fullScore = false;                             // Partition entière ou sélection

      

      // Cherche les portées, on ne travaillera pas sur la dernière portée (en général clé de Fa, Basses et Accords)

      var nbPortees = myScore.nstaves

      //console.log("Nombre de portées =",nbPortees)  

      

      // pas d'accord main gauche à priori

      accordMg = "zzz"   

      

      // ---------------------------------------------------------

      // Boucle sur chacune des portées sauf la dernière s'il y en a plusieurs

      // ---------------------------------------------------------

      do {

            cursor.voice    = 0;                             // on ne traite que la première voix

            cursor.staffIdx = staff;                       // on traite portée par portée 

          //  cursor.rewind(Cursor.SCORE_START); // rembobine au début de la partition

         cursor.rewind(1)

            if (!cursor.segment) { // pas de sélection

                  fullScore = true;

                  startStaff = 0;                              // commence à la première mesure

                  endStaff = curScore.nstaves - 1; // et termine à la dernière

            } else {

                  startStaff = cursor.staffIdx;        // commence au début de la sélection

                  cursor.rewind(2);                       // passe derrière le dernier segment et fixe tick = 0

                  if (cursor.tick === 0) {              // ceci survient lorsque la sélection contient la dernière mesure

                       endTick = curScore.lastSegment.tick + 1;

                  } else {

                      endTick = cursor.tick;

                 }

                 endStaff = cursor.staffIdx;

              }

              

              if (fullScore) {                             // pas de sélection

                  cursor.rewind(0);                   // rembobine au début de la partition

              } else {                                       // sélection

                 cursor.rewind(1);                    // rembobine au début de la sélection

              }

             // ------------------------------------------

             // Boucle pour chaque mesure de la portée en cours

             // ------------------------------------------

             //while(cursor.segment) {

             while (cursor.segment && (fullScore || cursor.tick < endTick))  {

                    var aCount = 0;

                   

                    // Recherche des accords main gauche (genre Am ou Em ou E7 ...)

                    var annotation = cursor.segment.annotations[aCount];

                    while (annotation) {

                           if (annotation.type == Element.HARMONY){                               

                                //console.log("Annotation : " + annotation.text);

                                accordMg = annotation.text

                           }

                           annotation = cursor.segment.annotations[++aCount];     

                    }

                    

                  // Si le curseur pointe sur une à plusieurs notes jouées simultanément

                  if (cursor.element && cursor.element.type == Element.CHORD) {

                        var notes = cursor.element.notes                           

                        // Boucle pour chaque note joué en même temps

                        for (var i = 0; i < notes.length; i++) {

                            addTouche(cursor, notes[i], accordMg)

                        } // end for chaque note de l'accord

                  } // end if CHORD

                  

                  cursor.next() //Element suivant

                  

             } // fin du while cursor.segment et (fullScore || cursor.tick < endTick)

             

             staff+=1 // Portée suivante 

             

      } while ((staff < curScore.nstaves-1) && fullScore)  // fin du for chaque portée sauf si sélection

       // Rappel : on ne traite pas la dernière portée qui est probablement en clé de fa, 

       // avec basses et accords. Pour la traiter quand même, il suffit de la sélectionner

       

     // Fin du plugin

      Qt.quit()

         

   } // onRun

} // MuseScore


